@@ -64,6 +64,7 @@ clearlooks_set_widget_parameters (const GtkWidget      *widget,
                                   WidgetParameters     *params)
 {
 	params->style_functions = &(clearlooks_style_class->style_functions[CLEARLOOKS_STYLE (style)->style]);
+	params->style_constants = &(clearlooks_style_class->style_constants[CLEARLOOKS_STYLE (style)->style]);
 
 	params->active        = (state_type == GTK_STATE_ACTIVE);
 	params->prelight      = (state_type == GTK_STATE_PRELIGHT);
@@ -197,7 +198,7 @@ clearlooks_style_draw_shadow (DRAW_ARGS)
 			STYLE_FUNCTION (draw_statusbar) (cr, colors, &params,
 			                                 x, y, width, height);
 	}
-	else if (DETAIL ("frame"))
+	else if (DETAIL ("frame") || DETAIL ("calendar"))
 	{
 		WidgetParameters params;
 		FrameParameters  frame;
@@ -214,9 +215,15 @@ clearlooks_style_draw_shadow (DRAW_ARGS)
 	}
 	else if (DETAIL ("scrolled_window") || DETAIL ("viewport") || detail == NULL)
 	{
-		CairoColor *border = (CairoColor*)&colors->shade[5];
+		CairoColor border;
+
+		if (CLEARLOOKS_STYLE (style)->style == CL_STYLE_CLASSIC)
+			ge_shade_color ((CairoColor*)&colors->bg[0], 0.78, &border);
+		else
+			border = colors->shade[5];
+
 		cairo_rectangle (cr, x+0.5, y+0.5, width-1, height-1);
-		ge_cairo_set_color (cr, border);
+		ge_cairo_set_color (cr, &border);
 		cairo_set_line_width (cr, 1);
 		cairo_stroke (cr);
 	}
@@ -487,7 +494,7 @@ clearlooks_style_draw_box (DRAW_ARGS)
 		gint columns, column_index;
 		gboolean resizable = TRUE;
 
-		/* XXX: This makes unknown treeview header CL_ORDER_MIDDLE, in need for something nicer */
+		/* XXX: This makes unknown treeview header "middle", in need for something nicer */
 		columns = 3;
 		column_index = 1;
 
@@ -509,12 +516,11 @@ clearlooks_style_draw_box (DRAW_ARGS)
 
 		header.resizable = resizable;
 
+		header.order = 0;
 		if (column_index == 0)
-			header.order = params.ltr ? CL_ORDER_FIRST : CL_ORDER_LAST;
-		else if (column_index == columns-1)
-			header.order = params.ltr ? CL_ORDER_LAST : CL_ORDER_FIRST;
-		else
-			header.order = CL_ORDER_MIDDLE;
+			header.order |= params.ltr ? CL_ORDER_FIRST : CL_ORDER_LAST;
+		if (column_index == columns-1)
+			header.order |= params.ltr ? CL_ORDER_LAST : CL_ORDER_FIRST;
 
 		gtk_style_apply_default_background (style, window, FALSE, state_type, area, x, y, width, height);
 
@@ -648,13 +654,16 @@ clearlooks_style_draw_box (DRAW_ARGS)
 	{
 		WidgetParameters params;
 		ScrollBarParameters scrollbar;
+		gboolean trough_under_steppers = TRUE;
+		ClearlooksStepper steppers;
 
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 		params.corners = CR_CORNER_ALL;
 
 		scrollbar.horizontal = TRUE;
 		scrollbar.junction   = clearlooks_scrollbar_get_junction (widget);
-
+		
+		steppers = clearlooks_scrollbar_visible_steppers (widget);
 
 		if (CHECK_HINT (GE_HINT_HSCROLLBAR))
 			scrollbar.horizontal = TRUE;
@@ -663,16 +672,42 @@ clearlooks_style_draw_box (DRAW_ARGS)
 		else /* Fallback based on the size  ... */
 			scrollbar.horizontal = width >= height;
 
-		/* What is this about? */
-		if (scrollbar.horizontal)
+		if (widget)
+			gtk_widget_style_get (widget,
+			                      "trough-under-steppers", &trough_under_steppers,
+			                      NULL);
+
+		if (trough_under_steppers)
 		{
-			x += 2;
-			width -= 4;
-		}
-		else
-		{
-			y += 2;
-			height -= 4;
+			/* If trough under steppers is set, then we decrease the size
+			 * slightly. The size is decreased so that the trough is not
+			 * visible underneath the steppers. This is not really needed
+			 * as one can use the trough-under-steppers style property,
+			 * but it needs to exist for backward compatibility. */
+			if (scrollbar.horizontal)
+			{
+				if (steppers & (CL_STEPPER_A | CL_STEPPER_B))
+				{
+					x += 2;
+					width -= 2;
+				}
+				if (steppers & (CL_STEPPER_C | CL_STEPPER_D))
+				{
+					width -= 2;
+				}
+			}
+			else
+			{
+				if (steppers & (CL_STEPPER_A | CL_STEPPER_B))
+				{
+					y += 2;
+					height -= 2;
+				}
+				if (steppers & (CL_STEPPER_C | CL_STEPPER_D))
+				{
+					height -= 2;
+				}
+			}
 		}
 
 		STYLE_FUNCTION(draw_scrollbar_trough) (cr, colors, &params, &scrollbar,
@@ -1299,6 +1334,16 @@ clearlooks_style_realize (GtkStyle * style)
 
 	ge_gdk_color_to_cairo (&style->bg[GTK_STATE_SELECTED], &spot_color);
 
+	/* Andrea Cimitan wants something like the following to handle dark themes.
+	 * However, these two lines are broken currently, as ge_hsb_from_color expects
+	 * a CairoColor and not GdkColor
+	 *  ge_hsb_from_color (&style->bg[GTK_STATE_SELECTED], &hue_spot, &saturation_spot, &brightness_spot);
+	 *  ge_hsb_from_color (&style->bg[GTK_STATE_NORMAL],   &hue_bg,   &saturation_bg,   &brightness_bg);
+	 */
+
+	/* Here to place some checks for dark themes.
+	 * We should use a different shade value for spot[2]. */
+
 	ge_shade_color (&spot_color, 1.25, &clearlooks_style->colors.spot[0]);
 	ge_shade_color (&spot_color, 1.05, &clearlooks_style->colors.spot[1]);
 	ge_shade_color (&spot_color, 0.65, &clearlooks_style->colors.spot[2]);
@@ -1733,13 +1778,23 @@ clearlooks_style_class_init (ClearlooksStyleClass * klass)
 	style_class->draw_layout      = clearlooks_style_draw_layout;
 	style_class->render_icon      = clearlooks_style_draw_render_icon;
 
-	clearlooks_register_style_classic (&clearlooks_style_class->style_functions[CL_STYLE_CLASSIC]);
+	clearlooks_register_style_classic (&clearlooks_style_class->style_functions[CL_STYLE_CLASSIC],
+	                                   &clearlooks_style_class->style_constants[CL_STYLE_CLASSIC]);
+
 	clearlooks_style_class->style_functions[CL_STYLE_GLOSSY] = clearlooks_style_class->style_functions[CL_STYLE_CLASSIC];
-	clearlooks_register_style_glossy (&clearlooks_style_class->style_functions[CL_STYLE_GLOSSY]);
+	clearlooks_style_class->style_constants[CL_STYLE_GLOSSY] = clearlooks_style_class->style_constants[CL_STYLE_CLASSIC];
+	clearlooks_register_style_glossy (&clearlooks_style_class->style_functions[CL_STYLE_GLOSSY],
+	                                  &clearlooks_style_class->style_constants[CL_STYLE_GLOSSY]);
+
 	clearlooks_style_class->style_functions[CL_STYLE_INVERTED] = clearlooks_style_class->style_functions[CL_STYLE_CLASSIC];
-	clearlooks_register_style_inverted (&clearlooks_style_class->style_functions[CL_STYLE_INVERTED]);
+	clearlooks_style_class->style_constants[CL_STYLE_INVERTED] = clearlooks_style_class->style_constants[CL_STYLE_CLASSIC];
+	clearlooks_register_style_inverted (&clearlooks_style_class->style_functions[CL_STYLE_INVERTED],
+	                                    &clearlooks_style_class->style_constants[CL_STYLE_INVERTED]);
+
 	clearlooks_style_class->style_functions[CL_STYLE_GUMMY] = clearlooks_style_class->style_functions[CL_STYLE_CLASSIC];
-	clearlooks_register_style_gummy (&clearlooks_style_class->style_functions[CL_STYLE_GUMMY]);
+	clearlooks_style_class->style_constants[CL_STYLE_GUMMY] = clearlooks_style_class->style_constants[CL_STYLE_CLASSIC];
+	clearlooks_register_style_gummy (&clearlooks_style_class->style_functions[CL_STYLE_GUMMY],
+	                                 &clearlooks_style_class->style_constants[CL_STYLE_GUMMY]);
 }
 
 GType clearlooks_type_style = 0;
