@@ -26,6 +26,7 @@
 #include <ge-support.h>
 #include <cairo.h>
 #include <math.h>
+#include <stdio.h>
 
 #define LINE_OPACITY 0.4
 #define HANDLE_OPACITY 0.38
@@ -516,6 +517,7 @@ draw_slider (GtkStyle      *style,
 	/* This makes the slider overlay the stepper buttons at the sides. XXX */
 	if (widget && GE_IS_SCROLLBAR (widget)) {
 		GtkAdjustment *adjustment = GTK_RANGE (widget)->adjustment;
+
 		/* If all of these values are 0, don't do anything.  This fixes mozilla/galeon scrollbars. */
 		if (adjustment->value ||
 		    adjustment->lower ||
@@ -523,19 +525,27 @@ draw_slider (GtkStyle      *style,
 		    adjustment->step_increment ||
 		    adjustment->page_increment ||
 		    adjustment->page_size) {
-			if (adjustment->value <= adjustment->lower) {
+			if ((adjustment->value <= adjustment->lower) &&
+			    (GTK_RANGE (widget)->has_stepper_a || GTK_RANGE (widget)->has_stepper_b)) {
 				if (orientation == GTK_ORIENTATION_VERTICAL) {
-					y--;
+					if (!gtk_range_get_inverted (GTK_RANGE (widget)))
+						y--;
 					height++;
 				} else {
-					x--;
+					if (!gtk_range_get_inverted (GTK_RANGE (widget)))
+						x--;
 					width++;
 				}
 			}
-			if (adjustment->value >= adjustment->upper - adjustment->page_size) {
+			if ((adjustment->value >= adjustment->upper - adjustment->page_size) &&
+			    (GTK_RANGE (widget)->has_stepper_a || GTK_RANGE (widget)->has_stepper_b)) {
 				if (orientation == GTK_ORIENTATION_VERTICAL) {
+					if (gtk_range_get_inverted (GTK_RANGE (widget)))
+						y--;
 					height++;
 				} else {
+					if (gtk_range_get_inverted (GTK_RANGE (widget)))
+						x--;
 					width++;
 				}
 			}
@@ -1118,7 +1128,6 @@ real_draw_box_gap (GtkStyle       *style,
 	/* clip the gap area */
 	cairo_save (cr);
 	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
-	cairo_rectangle (cr, x, y, width, height);
 
 	switch (gap_side) {
 	case GTK_POS_TOP:
@@ -1138,6 +1147,8 @@ real_draw_box_gap (GtkStyle       *style,
 	ge_cairo_set_color (cr, &bg);
 	if (fill)
 		cairo_fill_preserve (cr);
+
+	cairo_rectangle (cr, x, y, width, height);
 	cairo_clip (cr);
 
 	/* reset fill rule */
